/*
 wave.c : Intel WAVE format generation functions

    Copyright (C) 2005 Risto Maki-Petays

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include <stdlib.h>
#include <stdio.h>
#include "types.h"
#include "wave.h"

#define FormatID 'fmt '   /* chunkID for Format Chunk. NOTE: There is a space at the end of this ID. */

typedef struct
{
  char           chunkID[4];
  long           chunkSize;

  short          wFormatTag;
  unsigned short wChannels;
  unsigned long  dwSamplesPerSec;
  unsigned long  dwAvgBytesPerSec;
  unsigned short wBlockAlign;
  unsigned short wBitsPerSample;

/* Note: there may be additional fields here, depending upon wFormatTag. */

} FormatChunk;

#define DataID 'data'  /* chunk ID for data Chunk */

typedef struct
{
  char           chunkID[4];
  long           chunkSize;

  unsigned char  waveformData[];
} DataChunk;

int
print_wave_header(Sample *sample)
{
  int i;
  FormatChunk fc;
  DataChunk dc;

  fc.chunkID[0]         = 'f';
  fc.chunkID[1]         = 'm';
  fc.chunkID[2]         = 't';
  fc.chunkID[3]         = ' ';
  fc.chunkSize          = sizeof(FormatChunk) - sizeof(long) - 4 * sizeof(char);
  fc.wFormatTag         = 1; // normal, not compressed
  fc.wChannels          = sample->channels;
  fc.wBitsPerSample     = sample->bits;
  fc.dwSamplesPerSec    = sample->sampleRate;
  fc.wBlockAlign        = fc.wChannels * (fc.wBitsPerSample / 8);
  fc.dwAvgBytesPerSec   = fc.dwSamplesPerSec * fc.wBlockAlign;

  dc.chunkID[0]         = 'd';
  dc.chunkID[1]         = 'a';
  dc.chunkID[2]         = 't';
  dc.chunkID[3]         = 'a';
  dc.chunkSize          = sample->size;

  printf("RIFF");       // main block           4
  printi(4 + sizeof(FormatChunk) + sizeof(DataChunk) + sample->size);
  printf("WAVE");       // wave header          4
  fwrite(&fc, 1, sizeof(FormatChunk), stdout);
  fwrite(&dc, 1, sizeof(DataChunk), stdout);
  return 0;
}


